/*************************************************
 gb/gbc mbc7 rom dump script

 MBC7 Mapper (Used by Kirby's Tilt n' Tumble, Command Master)
===========

Status: Partial support (only ROM banking supported at the moment)

The MBC7 mapper has 0x0200(?) bytes of RAM built in.

0000-1FFF - Probably enable/disable RAM
            In order to use this area bit 12 of the address be set.
            Values written: 00, 0A
2000-2FFF - Writing to this area selects the ROM bank to appear at
            4000-7FFF.
            In order to use this area bit 12 of the address be set.
            Values written: 01, 07, 01, 1C
3000-3FFF - Unknown
            In order to use this area bit 12 of the address be set.
            Values written: 00
4000-4FFF - Unknown
            In order to use this area bit 12 of the address be set.
            Values written: 00, 40, 3F
*************************************************/
function cpu_dump(d, rom_size, banksize)
{
	cpu_read (d, 0x0000, 0x4000); 
	for(local i = 1; i < rom_size/0x4000; i += 1)
	{
		//cpu_write(d, 0x3000, i>>8);
		//cpu_write(d, 0x2000, i&0xFF);
		cpu_write(d, 0x3000, i>>8, 0x2000, i&0xFF);
		cpu_read (d, 0x4000, 0x4000); 
	}
}

function cpu_erase(d, rom_size, flash_chip_type)
{
	local m_29f_flash = 0;
	local m_29lv_flash = 1;
	
	//local flash_chip_type = m_29lv_flash;
	//change to m_29f_flash here, if your gb 
	//flash chip is 29f(mx29f016,amd29f016,amd29f080...)
	
	//detect chip here
	if(flash_chip_type==m_29f_flash)
	{
		cpu_write(d,0x555,0xF0);

		cpu_write(d,0x555,0xAA);
		cpu_write(d,0x2AA,0x55);
		cpu_write(d,0x555,0x80);

		cpu_write(d,0x555,0xAA);
		cpu_write(d,0x2AA,0x55);
		cpu_write(d,0x555,0x10);
	}
	else{
	//erase mx29lv320 
	cpu_write(d,0xAAA,0xF0);

	cpu_write(d,0xAAA,0xAA);
	cpu_write(d,0x555,0x55);
	cpu_write(d,0xAAA,0x80);
	cpu_write(d,0xAAA,0xAA);
	cpu_write(d,0x555,0x55);
	cpu_write(d,0xAAA,0x10);	
	}
	rom_flash_cfg(d,flash_chip_type);
}
function cpu_flash(d, rom_size, banksize)
{
	rom_write (d, 0x0000, 0x4000);
	for(local i = 1; i < rom_size/0x4000; i += 1)
	{
		cpu_write(d, 0x3000, i>>8, 0x2000, i&0xFF);
		rom_write (d, 0x4000, 0x4000);
	}
}
function save_dump(d, save_size, max_size)
{
	cpu_write (d, 0x0000, 0x0A); 
	for(local i = 0; i < save_size/0x2000; i += 1)
	{
		cpu_write(d, 0x4000, i);
		cpu_read (d, 0xA000, 0x2000); 
	}
	cpu_write (d, 0x0000, 0x00); 
}
function save_erase(d, save_size, max_size)
{
	//not need here
}
function save_write(d, save_size, max_size)
{
	cpu_write (d, 0x0000, 0x0A); 
	for(local i = 0; i < save_size/0x2000; i += 1)
	{
		cpu_write(d, 0x4000, i);
		ram_write(d, 0xA000, 0x2000); 
	}
	cpu_write (d, 0x0000, 0x00); 
}
